// $Id: Example.v,v 

`include "Src/ExampleTimescale.v"

module Example(
	clock,
	reset_n,
	data_in,
	data_in_valid,
	data_out,
	data_out_valid
	);

	parameter WIDTH = 4;   // bit width
	parameter DEPTH = 16;   // depth of fifo

	output	[(WIDTH-1):0] data_out;
	output  data_out_valid;
	input	clock, reset_n;
   	input 	[(WIDTH-1):0]  data_in;
   	input	data_in_valid;

	reg [(WIDTH-1):0]    fifo [0:(DEPTH-1)];
	reg                  en_fifo[0:(DEPTH-1)];

   	// write pointer (points to first free slot)
	integer i;    // loop count

	// first stage of fifo is output
	assign data_out = fifo[DEPTH-1];
	assign data_out_valid = en_fifo[DEPTH-1];


	always @(posedge clock or negedge reset_n)   // trigger new evaluation
		begin
		if (reset_n == 0)   // it's a reset
		begin
			for (i=0; i<DEPTH; i=i+1)   // zero all valid entries
			begin
				fifo[i] 	= 'b0000;
				en_fifo[i] 	= 0;
			end
	    end
	    else   // it's a posedge of clock
	    begin
			en_fifo[0] <= data_in_valid;
			fifo[0] <= data_in;
			for (i=1; i<DEPTH; i=i+1)   // shift all valid entries
			begin
				fifo[i] <= fifo[i-1];
				en_fifo[i] <= en_fifo[i-1];
			end
		end
	end // always @ (posedge clock or negedge reset_n)

endmodule
