// ExampleCol.cpp: implementation of the ExampleCol class.
//
//////////////////////////////////////////////////////////////////////

#include "ExampleCol.h"
#include <SpvPlugin.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ExampleCol::ExampleCol()
{
}

ExampleCol::~ExampleCol()
{

}

void ExampleCol::Start()
{
	bool successGet = true;
	successGet = successGet && SD::GetSig(m_SigClock,"S_CLOCK");
	successGet = successGet && SD::GetSig(m_SigReset,"S_RST");

	if(successGet == false) return;

	m_PosClock.Init(m_SigClock, AtPos);
	m_NegClock.Init(m_SigClock, AtNeg);
	m_PosReset.Init(m_SigReset, AtPos);
	m_NegReset.Init(m_SigReset, AtNeg);

	StartTProc(this, (SPVPM)&ExampleCol::PacketRestartCompare);

	SpvTProcCtrl *procCtrl;
	procCtrl = StartTProc(this, (SPVPM)&ExampleCol::PacketInThread);
	m_ThreadCtrls.push_back(procCtrl);
	procCtrl= StartTProc(this, (SPVPM)&ExampleCol::PacketOutThread);
	m_ThreadCtrls.push_back(procCtrl);
}

void ExampleCol::PacketInThread()
{
	bool successGet = true;
	SpvSig sigDataIn, sigDataInValid;
	
	// Signal Definition
	successGet = successGet && SD::GetSig(sigDataIn,		"S_DATA_IN");
	successGet = successGet && SD::GetSig(sigDataInValid,	"S_DATA_IN_VALID");

	if(successGet == false) return;

	Wait(m_PosReset);
	// Starting to insert data to the fifo
	for(;;) {
		if(sigDataInValid.Uint() == 1)
			m_Fifo.push_back(sigDataIn.Uint());
		Wait(m_NegClock);
	}
}

void ExampleCol::PacketOutThread()
{
	unsigned dataCnt = 0;
	bool printGoodData = false;
	bool successGet = true;
	SpvSig sigDataOut, sigDataOutValid;
	
	// Signal Definition
	successGet = successGet && SD::GetSig(sigDataOut,		"S_DATA_OUT");
	successGet = successGet && SD::GetSig(sigDataOutValid,	"S_DATA_OUT_VALID");
	
	// User does not have to write this if he does not want to...
	SD::GetBool(printGoodData,		"B_PRINT_GOOD_COMPARE");

	if(successGet == false) return;

	Wait(m_PosReset);
	// Starting to compare data from the fifo
	for(;;) {
		if(sigDataOutValid.Uint() == 1) {
			if(m_Fifo.size() == 0) {
				SPV_OUT(<<"***ERROR: Compare failed because data in fifo is empty, on Time: "<< SpvSig::time<<endl);
			}
			else {
                unsigned expected = TransferFunc(m_Fifo.front());
				if(!Compare(expected, sigDataOut.Uint())) {
					SPV_OUT(<<"["<<SpvSig::time<<"] -ERROR- Comparator: Data["<<dataCnt<<"] failed Received="<<sigDataOut.Uint()<<" Different Expected="<<expected<<endl);
				}
				else if(printGoodData) {
					SPV_OUT(<<"["<<SpvSig::time<<"] Comparator: Compare OK, Sample #"<<dataCnt<<", Received="<<sigDataOut.Uint()<<endl);
				}
				m_Fifo.pop_front();
			}
			dataCnt++;
		}
		Wait(m_PosClock);
	}
}

unsigned ExampleCol::TransferFunc(unsigned input)
{
    return input;
}

bool ExampleCol::Compare(unsigned expected, unsigned received)
{
    return expected == received;
}

void ExampleCol::PacketRestartCompare()
{
	for(;;) {
		Wait(m_NegReset);
        m_Fifo.clear();

		for(int i = 0; i < m_ThreadCtrls.size(); i++){
			m_ThreadCtrls[i]->Restart();
		}
	}
}
