// ExampleDriver.cpp: implementation of the ExampleDriver class.
//
//////////////////////////////////////////////////////////////////////

#include "ExampleDriver.h"
#include "ExamplePacket.h"
#include <SpvPlugin.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ExampleDriver::ExampleDriver()
{

}

ExampleDriver::~ExampleDriver()
{

}

void ExampleDriver::Start()
{	
	bool successGet = true;
	successGet = successGet && SD::GetSig(m_SigClock,"S_CLOCK");
	successGet = successGet && SD::GetSig(m_SigReset,"S_RST");

	if(successGet == false) return;

	// Init the events
	m_PosClock.Init(m_SigClock, AtPos);
	m_PosReset.Init(m_SigReset, AtPos);
	m_NegReset.Init(m_SigReset, AtNeg);

	StartTProc(this, (SPVPM)&ExampleDriver::ThreadReset);

	StartTProc(this, (SPVPM)&ExampleDriver::ThreadRestartData);

	SpvTProcCtrl *procCtrl = StartTProc(this, (SPVPM)&ExampleDriver::ThreadDriveData);
	m_ThreadCtrls.push_back(procCtrl);
}

void ExampleDriver::ThreadDriveData() 
{
	bool successGet = true;
	SpvSig sigDataIn, sigDataInValid;
	SpvGen *dataInGen, *dataEnGen;
	int	dataLength;
	
	// Signal Definition
	successGet = successGet && SD::GetSig(sigDataIn,		"S_DATA_IN");
	successGet = successGet && SD::GetSig(sigDataInValid,	"S_DATA_IN_VALID");

	// Generators Definition
	successGet = successGet && SD::GetGen(dataInGen,		"G_DATA_IN");
	successGet = successGet && SD::GetGen(dataEnGen,		"G_DATA_EN");

	// Constants Definitions
	successGet = successGet && SD::GetInt(dataLength,		"N_DATA_LENGTH");

	if(successGet == false) return;	

	sigDataInValid	= 0;
	sigDataIn		= 0;

	Wait(m_PosReset);

	// This function will drive Generated data in to the DUT
	for(unsigned i = 0; i < dataLength;) {
		unsigned dataEnCnt = dataEnGen->Gen();

		// This while loop will Generate the amount of clocks that valid should be up
		while(dataEnCnt-- > 0) {
			sigDataInValid	= 1;
			sigDataIn		= dataInGen->Gen();
			i++;
			Wait(m_PosClock);
		}

		sigDataInValid	= 0;
		Wait(m_PosClock);
	}

	Wait(m_PosClock);
}

void ExampleDriver::ThreadReset()
{
	// Waiting 2 Clocks, Reseting for 2 clocks and continue
	Wait(m_PosClock,2);
	m_SigReset	= 0;
	Wait(m_PosClock,2);
	m_SigReset	= 1;
}
	

// Restart function that listen for a reset event	
void ExampleDriver::ThreadRestartData()
{
	for(;;) {
		Wait(m_NegReset);
		for(int i = 0; i < m_ThreadCtrls.size(); i++){
			m_ThreadCtrls[i]->Restart();
		}
	}
}

